package com.ibm.mqe.model;

import com.ibm.mqe.*;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.administration.*;
import com.ibm.uia.renderingInterface.*;
/**
 * Insert the type's description here.
 * Creation date: (10/5/2000 7:53:48 AM)
 * @author: 
 */
abstract class AbstractQueue extends ManagedObject {
/**
 * performOperation method comment.
 */
public MQeAdminMsg addAlias(String aliasName) {
	MQeAdminMsg replyMsg = null;
	MQeQueueAdminMsg msg = queueAdminMessage();
	try {
		msg.addAlias(aliasName);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		//_log().println("Error creating queue alias");
		//e.printStackTrace(_log());
	}
	return replyMsg;
}
/**
 * reply an admin message for the MQe resource 
 * represented by the receiver.
 */
public final MQeAdminMsg adminMessage() {
	MQeQueueAdminMsg msg = queueAdminMessage();
	return msg;
}
/**
 * browse messages assciated with the receiver
 */
public MQeEnumeration browse() {
	return getQueueManager().browseQueue(getQueueQueueManagerName(), getName());
}
public boolean deleteMessage(MQeFields msg) {
	return getQueueManager().deleteMessage(getQueueQueueManagerName(), getName(), msg);
}
/**
 * Read the receivers data from the output data field
 */
public final String[] getAliasData(MQeFields data) {
	String[] alii = null;
	try {
		alii = data.getAsciiArray(MQeQueueAdminMsg.Queue_QAliasNameList);
	} catch (Exception e) {
		_comment("No aliases found");
	}
	return alii;
}
/**
 * Reply the name of the queue manager associated with the
 * receiver.  For most queues this is the name of the owning
 * queue manager.  For remote queues it is the name of the remote
 * queue manager that owns the real queue.  (The queue manager
 * where the queue is LOCAL). 
 */
String getQueueQueueManagerName() {
	return getQueueManager().getName();
}
/**
 * commit the receiver and its attributes
 * reply true if any change occurs
 */
protected boolean performCreate() {
	if (isToBeCreated()) {
		setStateCreating();
	}
	MQeAdminMsg replyMsg = null;
	try {
		MQeQueueAdminMsg msg = queueAdminMessage();
		msg.create(new MQeFields());
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		
	}
	return handleMessage(replyMsg);
}
protected boolean performDelete() {
	super.performDelete();
	MQeAdminMsg replyMsg = null;
	try {
		MQeQueueAdminMsg msg = queueAdminMessage();
		msg.delete(new MQeFields());
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		//_log().println("Error deleting queue");
		//e.printStackTrace(_log());
	}
	return handleMessage(replyMsg); 
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if ("Test Message".equals(command)) {
		performTestMessage(renderer);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
/**
 * Perform the placing of a tespt message.
 */
public boolean performTestMessage(Renderer renderer) {
	String[] data = renderer.promptUserForStrings("Enter TestMessage Text", new String[] {"text"}, new String[] {"Test Data"});
	if (null != data && data.length > 0) {
		testMessage(getQueueQueueManagerName(), getName(), data[0]);
	}
	return false;
}
/**
 * reply an admin message for the MQe resource 
 * represented by the receiver.
 */
public abstract MQeQueueAdminMsg queueAdminMessage();
/**
 * performOperation method comment.
 */
public MQeAdminMsg removeAlias(String aliasName) {
	MQeAdminMsg replyMsg = null;
	MQeQueueAdminMsg msg = queueAdminMessage();
	try {
		msg.removeAlias(aliasName);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		//_log().println("Error creating queue alias");
		//e.printStackTrace(_log());
	}
	return replyMsg;
}
/**
 * browse messages assciated with the receiver
 */
public void testMessage(String queueQueueManagerName, String queueName, String txt) {
	getQueueManager().testMessage(queueQueueManagerName, queueName, txt);
}
}
