package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
import com.ibm.uia.xml.*;
/**
 * This type is a description of permitted attributes and child
 * types for an object.
 */
public class MetaObject extends MetaModelComponent {
	private static final String CLASSNAME = "className";
	protected static final String METAOBJECT_TAG = "metaObject";
	protected static final String CATEGORY_TAG = "category";
	protected static final String AUTOADD_TAG = "autoAdd";
	protected static final String CHILD_TYPE = "childType";
	protected static final String CHILD_TYPE_NAME = "name";

	protected static final String MAPPING_TAG = "mapping";
	protected static final String MAPPING_NAME = "caption";
	protected static final String MAPPING_VALUE = "mappedValue";

	private boolean isCategory;
	private boolean autoAdd = true;
	private Vector operations;
	protected Hashtable attributeDefinitions;
	private Vector permittedChildTypes;
	private MetaModel metaModel;
	private Hashtable mappings;
	private String className = "com.ibm.uia.metamodel.UIObject";
/**
 * MetaObject constructor comment.
 * NOTE: we remove the attributes we use, so that
 * subclass implementations can ignore them
 */
public MetaObject(MetaModel metaModel) {
	super();
	attributeDefinitions = new Hashtable();
	mappings = new Hashtable();
	operations = new Vector();
	permittedChildTypes = new Vector();
	this.metaModel = metaModel;
}
/**
 * Reply an instance of UIObject with the given name and
 * initialised for the reciever.
 * This is intended to be the sole factory method for UIObjects, 
 * and allows subclasses of this type to substutute instances of
 * subclasses of UIObject into the model structure.
 */
protected UIObject createNewObject(String name) {
	UIObject uio = newObject();
	if (null != uio) {
		uio.initialise(name, this);
	}
	return uio;
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = super.generateDom(doc);
	Enumeration enum = attributeDefinitions.elements();
	while (enum.hasMoreElements()) {
		AttributeDefinition de = (AttributeDefinition)enum.nextElement();
		Element child = de.generateDom(doc);
		//el.appendChild(child);
	}
	enum = operations.elements();
	while (enum.hasMoreElements()) {
		Operation de = (Operation)enum.nextElement();
		Element child = de.generateDom(doc);
		el.appendChild(child);
	}
	enum = permittedChildTypes.elements();
	while (enum.hasMoreElements()) {
		Element child = doc.createElement(CHILD_TYPE);
		child.setAttribute(CHILD_TYPE_NAME, enum.nextElement().toString());
		el.appendChild(child);
	}
	enum = mappings.keys();
	while (enum.hasMoreElements()) {
		Element child = doc.createElement(MAPPING_TAG);
		String key = enum.nextElement().toString();
		child.setAttribute(MAPPING_VALUE, key);
		child.setAttribute(MAPPING_NAME, mappings.get(key).toString());
		el.appendChild(child);
	}
	
	return el;
}
/**
 * getAllOperations.
 * Reply an enumeration across the child types permitted for
 * the receiver.  Each member of the enumeration will be an
 * instance of MetaObject, or of one of its subclasses
 */
public final Enumeration getAllChildTypes() {
	return permittedChildTypes.elements();
}
/**
 * getAllOperations.
 * Reply an enumeration across the operations permitted for
 * the receiver.  Each member of the enumeration will be an
 * instance of Operation, or of one of its subclasses
 */
protected final Enumeration getAllOperations() {
	return operations.elements();
}
protected final MetaModel getMetaModel() {
	return metaModel;
}
final OperationDefinition getOperationDefinition(String name) {
	Enumeration enum = getAllOperations();
	while (enum.hasMoreElements()) {
		OperationDefinition opDef = (OperationDefinition)enum.nextElement();
		if (opDef.getName().equals(name)) {
			return opDef;
		}
	}
	return null;
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class
 */
public final String getTagName() {
	return METAOBJECT_TAG;
}
/**
 * Utility method to convert a string representation of 
 * a single unicode character into a string consisting of
 * only the unicode character.
 * EG the string "\u0030" would be converted to "0"
 */
private final String handleUnicode(String inStr) {
	// this oly works for a single unicaode character String
	if (!inStr.startsWith("\\u")) {
		return inStr;
	}
	char[] chars = new char[1];
	int i = Integer.parseInt(inStr.substring(2), 16);
	chars[0] = (char)i;
	return new String(chars);
}
/**
 * Reply true if the receiver was defined as 'auto add'
 *  This is only meaningful for MetaObjects defined as 
 * categories, and is used by the parent type.  When an
 * instance of UIObject is created, and has a permitted
 * child type that is both a category and auto add, then
 * an instance of UIObject representing the category will 
 * be created during construction of the parent.
 */
public final boolean isAutoAdd() {
	return autoAdd;
}
/**
 * reply true if the receiver has been declared to be a category
 */
public final boolean isCategory() {
	return isCategory;
}
/**
 * reply true if the supplied string is the name of a
 * type registered as a child type of the receiver.
 * If the receiver has child types which are categories, then
 * reply true if any of these categories has a child type with 
 * the specified name.  Note that searching of categories is
 * recursive to arbitrary depth
 */
public final boolean isValidChildType(String type) {
	// relies on 'contains()' using the 'equals()' method
	// which the documentation claims is true
	boolean allow = permittedChildTypes.contains(type);
	if (!allow) {
		// check for categories
		Enumeration enum = permittedChildTypes.elements();
		while (enum.hasMoreElements()) {
			MetaObject mo = getMetaModel().getMetaObject(enum.nextElement().toString());
			if (null != mo) {
				if (mo.isCategory) {
					if (mo.isValidChildType(type)) {
						return true;
					}
				}
			}
		}
	}
	return allow;
}
/**
 * Reply the mapped value for the given string, or the given
 * string if no mapping is available
 */
public final String mapValue(String val) {
	String mapped = (String)mappings.get(val);
	if (null == mapped) {
		mapped = val;
	}
	return mapped;
}
/**
 * reply a new instance of attribute initialised for the specified
 * AttributeDefinition and UIObject.
 * This method is called during the initialisation of an instance of
 * UIObject by the receiver.
 * This method is the intended factory method for attributes, and
 * is designed to allow subclasses to substitute instances of
 * subclasses of Attribute into the UIObjects attribute dictionary
 */
protected Attribute newAttribute(AttributeDefinition attDef, UIObject owner) {
	return new Attribute(attDef, owner);
}
/**
 * reply a new instance of attribute definition for the receiver.
 * This method is called during the initialisation of an instance of
 * this class from the metamodel xml.
 * This method is the intended factory method for attributeDefinitionss, and
 * is designed to allow subclasses of this type to substitute instances of
 * subclasses of AttributeDefinition into the receivers attribute definition
 * dictionary
 */
protected AttributeDefinition newAttributeDefinition() {
	return new AttributeDefinition(this);
}
/**
 * reply a new collection of attributes for the specified UIObject.
 * The attributes are based on the attribute definitions held by the 
 * receiver.
 * This method is caled by instances of UIObject during their initialisation.
 */
final Hashtable newAttributeDictionary(UIObject owner) {
	Hashtable dict = new Hashtable();
	Enumeration enum = attributeDefinitions.elements();
	while (enum.hasMoreElements()) {
		AttributeDefinition attDef = (AttributeDefinition)enum.nextElement();
		Attribute att = newAttribute(attDef, owner);
		dict.put(attDef.getName(), att);
	}
	return dict;
}
/**
 * Reply an instance of UIObject.
 * This is intended to be the sole factory method for UIObjects, 
 * and allows subclasses of this type to substutute instances of
 * subclasses of UIObject into the model structure.
 */
protected UIObject newObject() {
	UIObject uio = null;
	try {
		uio = (UIObject)MetaModel.InstanceFromName(className);
	} catch (ClassCastException e) {
		System.out.println("ERROR: Class " + className + " is not a subclass of UIObject");
	}
	return uio;
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected UIAComponent parseChildElement(Element element) {
	String elemName = element.getNodeName();
	if (AttributeDefinition.ATTRIBUTE_DEFINITION.equals(elemName)) {
		AttributeDefinition attDef = newAttributeDefinition();
		attDef.initialiseFrom(element);
		attributeDefinitions.put(attDef.getName(), attDef);
	} else if (OperationDefinition.OPERATION_TAG_TEXT.equals(elemName)) {
		OperationDefinition opDef = new OperationDefinition();
		opDef.initialiseFrom(element);
		operations.addElement(opDef);
	} else if (CHILD_TYPE.equals(elemName)) {
		permittedChildTypes.addElement(element.getAttribute(CHILD_TYPE_NAME));
	} else if (MAPPING_TAG.equals(elemName)) {
		String mapName = element.getAttribute(MAPPING_NAME);
		String mapValue = element.getAttribute(MAPPING_VALUE);
		mapValue = handleUnicode(mapValue);
		mappings.put(mapValue, mapName);
	} else {
		return super.parseChildElement(element);
	}
	return null;
}
/**
 * Initialise the receiver from the given Element.
 */ 
protected void parseDom(Element element) {
	super.parseDom(element);
	String str = element.getAttribute(CATEGORY_TAG);
	if (null != str) {
		isCategory = str.equalsIgnoreCase("true");
		element.removeAttribute(CATEGORY_TAG);
	}
	str = element.getAttribute(AUTOADD_TAG);
	if (null != str) {
		autoAdd = str.equalsIgnoreCase("true");
		element.removeAttribute(AUTOADD_TAG);
	}
	str = element.getAttribute(CLASSNAME);
	if (null != str) {
		className = str;
	}
	metaModel._register(this);
}
/**
 * Reply the unmapped value for the given string, or the given
 * string if no mapping is available
 */
public final String unmapValue(String val) {
	if (mappings.contains(val)) {
		Enumeration enum = mappings.keys();
		while (enum.hasMoreElements()) {
			String k = enum.nextElement().toString();
			if (mappings.get(k).equals(val)) {
				return k;
			}
		}
	}
	return val;
}
}
