package com.ibm.uia.renderingInterface;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.util.*;
/**
 * This interface marks a class as renderable.
 */
public interface RenderableObject extends Renderable {
/**
 * Place a representation of the receiver onto the system clipbard.
 * Reply true if successful
 */
public abstract boolean copy();
/**
 * Place a representation of the receiver onto the system clipbard.
 * If succesful, then delete the receiver.
 * Reply true if successful.
 */
public abstract boolean cut();
/**
 * Reply an enumeration across all the attributes of the receiver
 * All members of the enumeration must implement <code>RenderableAttribute</code>.
 * The definition of 'attribute' is a matter for the discretion of
 * the implementor of the interface.  Attributes represent a value 
 * associated with the receiver (attribute value) and distinguished 
 * from other attributes by a unique (for the receiver) tag (the 
 * attribute name).
 * All members of the enumeration must be instances of RenderableAttribute.
 * If the receiver has no attributes, an empty enumeration will be returned.
*/
public abstract Enumeration getAllAttributes();
/**
 * Reply an enumeration across all the children of the receiver.
 * All members of the enumeration must implement <code>RenderableObject</code>.
 * The definition of 'child' is a matter for the discretion of
 * the implementor of the interface.  Children will be displayed
 * immediately below the receiver in any hierarchical (tree) view.
 * All members of the enumeration must be instances of RenderableObject.
 * If the receiver has no children, an empty enumeration will be returned.
 */
public abstract Enumeration getAllChildren();
/**
 * Reply an enumeration across all the operations of the receiver.
 * All the members of the enumeration must implement <code>RenderableOperation</code>.
 * The definition of 'operation' is a matter for the discretion of
 * the implementor of the interface.  Operations represent represent
 * actions associated with the receiver.  Each operation must have a 
 * unique identity.
 * If the receiver has no operations, an empty enumeration will be returned.
 */
public abstract Enumeration getAllOperations();
public abstract java.awt.Color getDisplayColour();
/**
 * Reply true if the supplied renderable operation is currently enabled 
 * for the receiver.  Reply false if the operation is not currently
 * enabled.
 */
public abstract boolean isOperationEnabled(RenderableOperation rop);
/**
 * Reply true if the supplied renderable operation is currently ticked 
 * for the receiver.  Reply false if the operation is not currently
 * tickeded.
 */
public abstract boolean isOperationTicked(RenderableOperation rop);
/**
 * Access and parse the daa on the system clipboard.
 * reply true if succesful
 */
public abstract boolean paste();
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * 
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is reqired.
 */
public abstract boolean performOperation(RenderableOperation op, Renderer renderer);
/**
 * Set the renderer used by the receiver.  
 * This is useful for asynchronous updates (ie those NOT generated 
 * through the renderer).
 * This method is only guaranteed to be called for UIObjects initially
 * rendered by the renderer, and not for the children of the UIObject.
 */
public abstract void setRenderer(Renderer renderer);
}
